@@ -77,7 +77,11 @@ class DeskbarApplet:
 			self.ui.connect('show-preferences', self.on_preferences, None)
 			self.ui.connect('show-about', self.on_about, None)
 			self.ui.connect('clear-history', self.on_clear_history, None)
-		
+		else:
+			# How can this happen ?? Apparently a bug report had this problem
+			# I check and recheck, i can't figure out a way that ui_name is unset..
+			self.ui = CuemiacButtonUI (applet, self.prefs)
+			
 		# Set up the chosen UI
 		self.set_up_ui_signals ()
 		self.ui.set_sensitive (False)
@@ -121,9 +125,12 @@ class DeskbarApplet:
 		match.action(text)
 		get_deskbar_history().add(text, match)
 		if self.clear_entry:
-			gobject.idle_add(self.clear_entry)
+			gobject.idle_add(self.do_clear_entry)
+		else:
+			self.ui.get_entry().set_position(-1)
+			self.ui.get_entry().select_region(0, -1)
 	
-	def clear_entry(self):
+	def do_clear_entry(self):
 		entry = self.ui.get_entry()
 		if entry != None:
 			entry.set_text("")
@@ -291,6 +298,11 @@ class DeskbarApplet:
 		return False
 	
 	def on_applet_button_release(self, widget, event):
+		# The check for middle_click != None is to "fix" bug 355687
+		if self.middle_click is None:
+			print "WARN, DeskbarApplet: self.middle_click is None on button release"
+			return
+			
 		if event.button == 2:
 			x, y = self.middle_click
 			diffx, diffy = abs(x-event.x_root), abs(y-event.y_root)
